package main

import (
	"bufio"
	"errors"
	"fmt"
	"io"
	"math/rand"
	"os"
	"strconv"
	"strings"
	"time"
)

type Node struct {
	X    int
	Y    int
	Next *Node
}

var size = 0
var queue = new(Node)

func Push(t *Node, v1, v2 int) bool {
	if queue == nil {
		queue = &Node{v1, v2, nil}
		size++
		return true
	}

	t = &Node{v1, v2, nil}
	t.Next = queue
	queue = t
	size++

	return true
}

func Pop(t *Node) (int, int, bool) {
	if size == 0 {
		return 0, 0, false
	}

	if size == 1 {
		queue = nil
		size--
		return t.X, t.Y, true
	}

	temp := t
	for (t.Next) != nil {
		temp = t
		t = t.Next
	}

	v1 := (temp.Next).X
	v2 := (temp.Next).Y
	temp.Next = nil

	size--
	return v1, v2, true
}

func loadMaze(file string) ([][]int, error) {
	var err error
	var mySlice = make([][]int, 0)

	f, err := os.Open(file)
	if err != nil {
		return nil, err
	}
	defer f.Close()

	r := bufio.NewReader(f)
	for {
		line, err := r.ReadString('\n')
		fields := strings.Fields(line)
		temp := make([]int, 0)
		for _, v := range fields {
			n, err := strconv.Atoi(v)
			if err != nil {
				return nil, err
			}
			temp = append(temp, n)
		}
		if len(temp) != 0 {
			mySlice = append(mySlice, temp)
		}

		if err == io.EOF {
			break
		} else if err != nil {
			return nil, err
		}

		if len(temp) != len(mySlice[0]) {
			return nil, errors.New("Wrong number of elements!")
		}
	}
	return mySlice, nil
}

func printSlice(x [][]int) {
	for _, v := range x {
		for _, c := range v {
			fmt.Print(c, " ")
		}
		fmt.Println()
	}
}

func getAdj(maze, v [][]int, x, y int) (int, int) {
	if x-1 >= 0 && v[x-1][y] == 0 && maze[x-1][y] == 1 { // left
		v[x-1][y] = 1
		return x - 1, y
	}
	if x+1 < len(v) && v[x+1][y] == 0 && maze[x+1][y] == 1 { // right
		v[x+1][y] = 1
		return x + 1, y
	}
	if y-1 >= 0 && v[x][y-1] == 0 && maze[x][y-1] == 1 { // up
		v[x][y-1] = 1
		return x, y - 1
	}
	if y+1 < len(v[0]) && v[x][y+1] == 0 && maze[x][y+1] == 1 { // down
		v[x][y+1] = 1
		return x, y + 1
	}
	return -1, -1
}

func BFS(maze, visited [][]int, x1, y1, x2, y2 int) bool {
	visited[x1][y1] = 1
	if maze[x1][y1] == 0 {
		return false
	}
	Push(queue, x1, y1)

	for {
		if size == 0 {
			return false
		}

		xx1, yy1, _ := Pop(queue)
		fmt.Print(" -> (", xx1, ",", yy1, ") ")
		if xx1 == x2 && yy1 == y2 {
			return true
		}

		for {
			t1, t2 := getAdj(maze, visited, xx1, yy1)
			if t1 == -1 || t2 == -1 {
				break
			} else {
				visited[t1][t2] = 1
				Push(queue, t1, t2)
				xx1 = t1
				yy1 = t2
			}
		}
	}
	return false
}

func main() {
	arguments := os.Args
	if len(arguments) == 1 {
		fmt.Printf("usage: solveMaze textFile\n")
		return
	}
	file := arguments[1]
	myMaze, err := loadMaze(file)
	if err != nil {
		fmt.Println(err)
		return
	}

	printSlice(myMaze)

	visited, _ := loadMaze(file)
	for x, v := range visited {
		for y, _ := range v {
			visited[x][y] = 0
		}
	}

	queue = nil

	rand.Seed(time.Now().UnixNano())
	x1 := rand.Intn(len(myMaze))
	y1 := rand.Intn(len(myMaze[0]))
	x2 := rand.Intn(len(myMaze))
	y2 := rand.Intn(len(myMaze[0]))

	fmt.Println("From:", x1, y1, "To:", x2, y2)
	if BFS(myMaze, visited, x1, y1, x2, y2) {
		fmt.Println("\nMaze is solvable!")
	} else {
		fmt.Println("\nMaze is NOT solvable!")
	}
}
